#!/usr/bin/env python3
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
import time

import numpy
import pytest

from braintech.obci.signal_processing.signal.data_raw_write_proxy import DataRawWriteProxy
from braintech.obci.signal_processing.signal.data_raw_write_proxy import SamplePacket
from braintech.obci.signal_processing.signal.data_read_proxy import DataReadProxy
from braintech.obci.signal_processing.signal.signal_exceptions import NoNextValue


def test_data_file_proxy(tmpdir):
    f_name = str(tmpdir.join('./tescik.obci.dat'))
    px = DataRawWriteProxy(f_name)

    def write_single_value(value):
        packet = SamplePacket(ts=numpy.array([time.time()]), samples=numpy.array([[value]]))
        px.data_received(packet)

    write_single_value(1.2)

    write_single_value(0.0023)

    write_single_value(-123.456)

    write_single_value(3.3)

    write_single_value(5.0)

    assert px.finish_saving() == (f_name, 5)

    py = DataReadProxy(f_name)

    assert abs(1.2 - py.get_next_value()) < 0.0001

    assert abs(0.0023 - py.get_next_value()) < 0.000000001

    assert abs(-123.456 - py.get_next_value()) < 0.00001

    py.goto_value(1)

    assert abs(0.0023 - py.get_next_value()) < 0.000000001

    py.goto_value(4)

    assert abs(5.0 - py.get_next_value()) < 0.0001

    py.goto_value(6)

    with pytest.raises(NoNextValue):
        py.get_next_value()

    py.finish_reading()

    py.start_reading()

    vs = py.get_next_values(3)

    assert abs(vs[0] - 1.20000000e+00) + abs(vs[1] - 2.30000000e-03) + abs(vs[2] + 1.23456000e+02) < 3 * 0.0001

    with pytest.raises(NoNextValue):
        py.get_next_values(3)
    # warning here

    py.finish_reading()
