#!/usr/bin/env python3
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

from braintech.obci.signal_processing.tags import tags_file_reader as t
from braintech.obci.signal_processing.tags import tags_file_writer as p


def test_file_tags(tmpdir):
    f_name = str(tmpdir.join('tescik.obci.tags'))
    px = p.TagsFileWriter(f_name)

    px.tag_received({'start_timestamp': 1001.0, 'end_timestamp': 1002.0, 'name': 'nic', 'channels': 'A B C',
                     'desc': {'x': 123, 'y': 456, 'z': 789}})

    px.tag_received({'start_timestamp': 1003.0, 'end_timestamp': 1004.0, 'name': 'nic2', 'channels': 'A B C',
                     'desc': {'x': 1234, 'y': 4567, 'z': 789}})

    px.tag_received({'start_timestamp': 1005.0, 'end_timestamp': 1006.0, 'name': 'nic3', 'channels': 'A B C',
                     'desc': {'x': 12345, 'y': 45678, 'z': 789}})

    px.set_first_sample_timestamp(1000.0)
    assert px.save_tags() == f_name

    py = t.TagsFileReader(f_name)

    g = py.get_tags()[0]

    assert g['start_timestamp'] == 1.0

    assert g['end_timestamp'] == 2.0

    assert g['name'] == 'nic'

    assert g['desc']['y'] == '456'

    assert [int(t['start_timestamp']) for t in py.get_tags()] == [1, 3, 5]

    # REORDER AND TEST ORDERING AGAIN ....

    px = p.TagsFileWriter(f_name)

    px.tag_received({'start_timestamp': 1003.0, 'end_timestamp': 1004.0, 'name': 'nic2', 'channels': 'A B C',
                     'desc': {'x': 1234, 'y': 4567, 'z': 789}})

    px.tag_received({'start_timestamp': 1005.0, 'end_timestamp': 1006.0, 'name': 'nic3', 'channels': 'A B C',
                     'desc': {'x': 12345, 'y': 45678, 'z': 789}})

    px.tag_received({'start_timestamp': 1001.0, 'end_timestamp': 1002.0, 'name': 'nic', 'channels': 'A B C',
                     'desc': {'x': 123, 'y': 456, 'z': 789}})

    assert [int(t['start_timestamp']) for t in py.get_tags()] == [1, 3, 5]
