import numpy as np
import pytest

from braintech.obci.signal_processing.signal.data_generic_write_proxy import Impedance, NETWORK_FLOAT32


def flags_from_channel_names(channel_names):
    return np.array([
        Impedance.PRESENT if channel_name.startswith('EXG') else
        Impedance.UNKNOWN if channel_name.startswith('TMSI') else
        Impedance.NOT_APPLICABLE
        for idx, channel_name in enumerate(channel_names)
    ], dtype=Impedance.NETWORK_INT8)


def channel_ids(channel_names, channel_type):
    return list(
        map(
            lambda idx_and_bool: idx_and_bool[0],
            filter(
                lambda idx_and_name: idx_and_name[1],
                enumerate(
                    map(lambda val: val.startswith(channel_type), channel_names)
                )
            )
        )
    )


@pytest.fixture(
    params=[
        {
            'channels': [],
            'samples': Impedance.create_empty_data(0),
        },
        {
            'channels': ['TECH_1', 'TECH_2', 'TECH_3'],
            'samples': Impedance.create_empty_data(0),
        },
        {
            'channels': ['EXG_1', 'TECH_1', 'EXG_2', 'TMSI_1'],
            'samples': np.array([[1, 2], [3, 4]], dtype=NETWORK_FLOAT32),
        },
    ]
)
def data(request):
    return request.param


@pytest.fixture()
def impedance(data):
    return Impedance(flags_from_channel_names(data['channels']), data['samples'])


def test_impedance_for_channel(impedance, data):
    impedance_channels = channel_ids(data['channels'], 'EXG')
    if data['samples'] is not None:
        assert all(
            (impedance.for_channel(idx) == values).all()
            for idx, values in zip(impedance_channels, data['samples'].T)
        ), 'Impedance for channels should be correct if available'

    technical_channels = channel_ids(data['channels'], 'TECH')
    assert all(
        impedance.for_channel(idx) == Impedance.NOT_APPLICABLE
        for idx in technical_channels
    ), 'Impedance for technical channels should not be applicable'

    tmsi_channels = channel_ids(data['channels'], 'TMSI')
    assert all(
        impedance.for_channel(idx) == Impedance.UNKNOWN
        for idx in tmsi_channels
    ), 'Impedance for tmsi channels should be unknown'


def test_code_and_decode(impedance):
    assert Impedance.from_bytes(impedance.to_bytes(), impedance.channel_count, impedance.sample_count)
