# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import cv2

from . import head


COLOR = (0, 0, 255)


def annotate(image, shape, pose, euler_angle):
    image = _draw_feature_circles(image, shape)
    image = _draw_head_box_on_image(image, pose)
    image = _draw_euler_angle_text(image, euler_angle)
    return image


def _draw_feature_circles(image, shape):
    for part in shape.parts():
        cv2.circle(image, (part.x, part.y), 2, COLOR, -1)
    return image


def _draw_head_box_on_image(image, pose):
    lines = head.calculate_head_box(pose)
    for line in lines:
        cv2.line(image, line[0], line[1], COLOR)
    return image


def _draw_euler_angle_text(image, euler_angle):
    outtext = "X: " + str(euler_angle[0])
    cv2.putText(image, outtext, (50, 40), cv2.FONT_HERSHEY_SIMPLEX, 0.45, (0, 0, 0))
    outtext = "Y: " + str(euler_angle[1])
    cv2.putText(image, outtext, (50, 60), cv2.FONT_HERSHEY_SIMPLEX, 0.45, (0, 0, 0))
    outtext = "Z: " + str(euler_angle[2])
    cv2.putText(image, outtext, (50, 80), cv2.FONT_HERSHEY_SIMPLEX, 0.45, (0, 0, 0))
    return image
