# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import logging
import math

import dlib

from bci_artifacts.install import download_face_predictor, TRAINED_FACE_SHAPE_PREDICTOR_PATH
from . import settings

logger = logging.getLogger(__name__)


class FacesDetector:
    def __init__(self):
        self._faces_detector = dlib.get_frontal_face_detector()

    def detect(self, image):
        return self._faces_detector(image, 1)


class FaceShapePredictor:
    def __init__(self):
        self._predictor = self._get_predictor()

    def _get_predictor(self):
        if not TRAINED_FACE_SHAPE_PREDICTOR_PATH.exists():
            download_face_predictor()
        predictor_path = str(TRAINED_FACE_SHAPE_PREDICTOR_PATH)
        return dlib.shape_predictor(predictor_path)

    def predict(self, image, face):
        return self._predictor(image, face)


def pick_closest_to_center(faces):
    best = None
    best_distance = None
    picture_center_x = settings.RESOLUTION[0] / 2
    picture_center_y = settings.RESOLUTION[1] / 2
    for face in faces:
        center = face.center()
        distance_x = abs(picture_center_x - center.x)
        distance_y = abs(picture_center_y - center.y)
        distance = math.sqrt(distance_x ** 2 + distance_y ** 2)
        if best is None or distance < best_distance:
            best = face
            best_distance = distance
    return best
