# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
from braintech.obci.experiment.peer.configured_peer import ConfiguredPeer, SignalReceiverMixin
from braintech.obci.core.broker.messages import SignalMessage
from braintech.obci.core.utils.message_helpers import send_unpacked_tag
from braintech.obci.core.utils.properties import param_property, split_floats, join_strings, split_string
from .eeg_artifacts_detector import Thresholds, EEGArtifactsDetectorConfig, \
    EpochingEEGArtifactsDetector

__all__ = ('EEGArtifactsDetectorPeer',)


def make_threshold(s_string):
    if len(s_string) == 0:
        return None
    else:
        return Thresholds(*split_floats(s_string))


def parse_bool(f_string):
    f_string = f_string.lower()
    if f_string in ['1', 'on', 'true']:
        return True
    elif f_string in ['0', 'off', 'false']:
        return False
    raise ValueError("Wrong value:%s" % f_string)


class EEGArtifactsDetectorPeer(SignalReceiverMixin, ConfiguredPeer):
    ALLOW_EMPTY_SOURCE = False
    sampling_rate = param_property('sampling_rate', float)
    channel_names = param_property('channel_names', split_string, join_strings)
    channel_gains = param_property('channel_gains', split_floats, join_strings)
    channel_offsets = param_property('channel_offsets', split_floats, join_strings)

    epoch_duration = param_property('epoch_duration', float)
    """Signal epoch duration analyzed each time"""
    epoch_interval = param_property('epoch_interval', float)

    training_duration = param_property('training_duration', float)
    update_training = param_property('update_training', parse_bool)
    """Update training every 'training_duration' samples"""

    merge_tags = param_property('merge_tags', parse_bool)

    max_data_duration = param_property('max_data_duration', float)
    """Maximum amount of data used for training"""

    slope_window = param_property('slope_window', float)
    slow_window = param_property('slow_window', float)
    outliers_merge_window = param_property('outliers_merge_window', float)
    muscles_freq_range = param_property('muscles_freq_range', split_floats)
    slopes = param_property('slopes', make_threshold, join_strings)
    slows = param_property('slows', make_threshold, join_strings)
    outliers = param_property('outliers', make_threshold, join_strings)
    muscles = param_property('muscles', make_threshold, join_strings)

    _detector = None

    async def _signal_message_handler(self, msg: SignalMessage):
        if self._detector:
            tags = self._detector.handle_sample_packet(msg.data)
            if self.merge_tags:
                tags += self._detector.merge_tags(tags)
            for tag in tags:
                await send_unpacked_tag(self, tag)

    async def _start(self):
        self._init_detector()
        await super(EEGArtifactsDetectorPeer, self)._start()

    def _init_detector(self):
        config = EEGArtifactsDetectorConfig()
        for param in config.params():
            val = getattr(self, param.lower(), None)
            if val is not None:
                setattr(config, param.upper(), val)
        self._detector = EpochingEEGArtifactsDetector(
            sample_rate=self.sampling_rate,
            channel_names=self.channel_names,
            epoch_duration=self.epoch_duration,
            epoch_interval=self.epoch_interval,
            config=config,
            max_data_duration=self.max_data_duration,
            training_duration=self.training_duration,
            update_training=self.update_training,
        )
