# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

from unittest import mock

import pytest

from bci_artifacts.camera import face


@pytest.mark.incremental
class TestPickClosestToCenter:
    def test_none_on_no_faces(self):
        assert face.pick_closest_to_center([]) is None

    def test_single_face(self):
        _face = mock.Mock()
        center_point = mock.Mock()
        center_point.x = 0
        center_point.y = 0
        _face.center = mock.Mock(return_value=center_point)
        assert _face == face.pick_closest_to_center([_face])

    @mock.patch('bci_artifacts.camera.settings.RESOLUTION', new=(1000, 1000))
    def test_many_faces_same_plane(self):
        face1 = mock.Mock()
        center_point1 = mock.Mock()
        center_point1.x = 0
        center_point1.y = 500
        face1.center = mock.Mock(return_value=center_point1)
        face2 = mock.Mock()
        center_point2 = mock.Mock()
        center_point2.x = 500
        center_point2.y = 500
        face2.center = mock.Mock(return_value=center_point2)
        face3 = mock.Mock()
        center_point3 = mock.Mock()
        center_point3.x = 500
        center_point3.y = 1000
        face3.center = mock.Mock(return_value=center_point3)
        assert face2 == face.pick_closest_to_center([face1, face2])
        assert face2 == face.pick_closest_to_center([face2, face3])
        assert face2 == face.pick_closest_to_center([face1, face2, face3])

    @mock.patch('bci_artifacts.camera.settings.RESOLUTION', new=(1000, 1000))
    def test_many_faces_many_planes(self):
        face1 = mock.Mock()
        center_point1 = mock.Mock()
        center_point1.x = 0
        center_point1.y = 0
        face1.center = mock.Mock(return_value=center_point1)
        face2 = mock.Mock()
        center_point2 = mock.Mock()
        center_point2.x = 500
        center_point2.y = 500
        face2.center = mock.Mock(return_value=center_point2)
        face3 = mock.Mock()
        center_point3 = mock.Mock()
        center_point3.x = 1000
        center_point3.y = 1000
        face3.center = mock.Mock(return_value=center_point3)
        assert face2 == face.pick_closest_to_center([face1, face2])
        assert face2 == face.pick_closest_to_center([face2, face3])
        assert face2 == face.pick_closest_to_center([face1, face2, face3])
