# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
from pathlib import Path

from bci_demo_ssvep import config_generation
from bci_demo_ssvep.gui.ssvep_frontend import BlinkerController, SSVEPFrontend
from bci_demo_ssvep.peers.analysis import SSVEPAnalysisPeer
from bci_demo_ssvep.peers.logic import SSVEPLogic
from braintech.obci.core.broker.broker import Broker
from obci_demo.gui import bci_demo_app
from obci_demo.gui.bci_controller import BCIDemoController
from obci_demo.gui.bci_demo_app import P300BCIDemo
from obci_demo.gui.bci_frontend import OBCIDemoExperimentWithFrontend, BCIFrontend
from obci_demo.obci_demo_experiment import ANALYSIS_ID, LOGIC_ID, OBCIDemoExperiment
# noinspection PyUnresolvedReferences
from .gui import bci_demo_ssvep_resources_rc  # noqa


class SSVEPDemoExperiment(OBCIDemoExperimentWithFrontend):
    def __init__(self, config, frontend, blinker):
        self._blinker = blinker
        super().__init__(config, frontend)

    def _merged_config(self, config):
        conf = {}
        conf.update(config['amplifier'])
        conf.update(config['ssvep'])
        return conf

    def _create_analysis(self, config):
        return self._add_peer(self._as_subprocess, SSVEPAnalysisPeer,
                              peer_id=ANALYSIS_ID, config=self._merged_config(config))

    def _create_logic(self, config):
        return self._add_peer(False, SSVEPLogic,
                              peer_id=LOGIC_ID, config=self._merged_config(config),
                              frontend=self._frontend,
                              blinker=self._blinker)

    def _create_broker(self):
        return Broker()

    def is_calibrated(self):
        return self._logic.is_calibrated

    def key_pressed(self, key_name):
        self._frontend.handle_key(key_name)

    def start(self, config):
        super(SSVEPDemoExperiment, self).start(config)
        wisdom_path = config['ssvep']['wisdom_path']
        if Path(wisdom_path).exists():
            self._logic.load_calibration(wisdom_path)

    def generate_config(self, config):
        return config_generation.generate(
            wisdom_path=self._config['ssvep']['wisdom_path'],
            **config
        )


class SSVEPController(BCIDemoController):

    def __init__(self, config, frontend: BCIFrontend, experiment: OBCIDemoExperiment):
        super().__init__(config, frontend, experiment)


class SSVEPBCIDemo(P300BCIDemo):
    PREFIX = 'ssvep'
    WINDOW_TITLE = 'SSVEP Demo'

    def create_frontend(self):
        self._blinker = BlinkerController(2)
        return SSVEPFrontend(self._config['ssvep']['field_statuses'], self._blinker)

    def create_experiment(self):
        return SSVEPDemoExperiment(self._config, self.frontend, self._blinker)


def run(config):
    bci_demo_app.run(config, SSVEPBCIDemo)
