# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import asyncio
import threading
import time
from asyncio import CancelledError
from queue import Empty


class SimpleFeedbackLogic:
    QUESTION_TIME = 5
    FEEDBACK_TIME = 2

    def __init__(self, analysis, blinker_controller, frontend):
        self.analysis = analysis
        self.blinker_controller = blinker_controller
        self.frontend = frontend

    async def endless_feedback(self):
        while True:
            self.analysis.clear_prediction_queue()
            correct_answer = await self.frontend.ask_question()
            self.blinker_controller.start_stimulation()
            stop_event = threading.Event()
            try:
                prediction = await asyncio.get_event_loop().run_in_executor(None, self._try_to_get_answer, stop_event,
                                                                            10.0)
            except CancelledError:
                stop_event.set()
                raise

            if prediction is None:
                await self.handle_no_prediction()
            else:
                is_correct = correct_answer is None or correct_answer == prediction
                await self.handle_prediction(prediction, is_correct)
                await self.frontend.wait_for(self.frontend.SPACE)

    def _try_to_get_answer(self, stop_event, max_wait):
        start = time.monotonic()
        prediction = None
        while prediction is None and not stop_event.is_set() and time.monotonic() - start < max_wait:
            try:
                prediction = self.analysis.get_prediction_field(block=True, timeout=0.5)
            except Empty:
                pass
        return prediction

    async def handle_no_prediction(self):
        await self.frontend.display_no_prediction()

    async def handle_prediction(self, field, is_correct):
        self.blinker_controller.fullbright()
        await self.frontend.display_prediction(field, self.FEEDBACK_TIME, is_correct)
