# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import asyncio
from logging import getLogger

LOGGER = getLogger(__name__)


class Frontend:
    SPACE = ' '
    F_KEY = 'f'

    def __init__(self, loop=None):
        self.set_loop(loop)

    def set_loop(self, loop):
        self._loop = loop
        self._f_pressed = asyncio.Event(loop=loop)
        self._space_pressed = asyncio.Event(loop=loop)

    def handle_key(self, key):
        LOGGER.debug("Handle key '%s'", key)
        if key == self.F_KEY:
            self._f_pressed.set()
        elif key == self.SPACE:
            self._space_pressed.set()

    def _get_key(self, key):
        return self._space_pressed if key == self.SPACE else self._f_pressed

    async def wait_for(self, key, clear=True):
        key = self._get_key(key)
        if clear:
            key.clear()
        await key.wait()
        key.clear()

    def was_pressed(self, key):
        key = self._get_key(key)
        is_set = key.is_set()
        key.clear()
        return is_set

    def was_space_pressed(self):
        return self.was_pressed(self.SPACE)

    def was_f_pressed(self):
        return self.was_pressed(self.F_KEY)

    async def display_no_prediction(self):
        LOGGER.debug("No prediction")

    async def display_prediction(self, field, feedback_duration, is_correct=True):
        LOGGER.debug("Prediction: %s is correct:%s", field, is_correct)
        await asyncio.sleep(feedback_duration)

    async def ask_question(self):
        return None

    async def calibrate_for(self, field_id):
        LOGGER.debug("Calibrate for %s", field_id)

    async def calibration_finished(self):
        pass

    async def prepare_calibration(self, wait_on_space):
        pass
