import os
from pathlib import Path

from setuptools import find_packages, setup, Command
from setuptools.command.sdist import sdist
import versioneer

INSTALL_REQUIREMENTS = {
    'install': [
        'braintech-obci-core~=2.8.0',
        'coloredlogs==10.0',
        'braintech-drivers-native-amplifier-lib~=2.8.0',
        'obci_demo',
        'obci_p300',
        'bci-ssvep',
        'obci_ugm',
    ],
    'setup': [
        'setuptools_scm'
    ]
}

try:
    from devops.setup import get_cmd_class

    cmd_classes = get_cmd_class()
except ImportError:
    print("devops not installed yet, can't run build_ui.")
    cmd_classes = {}


def package_bci_demo():
    from fabric import api
    from fabric.context_managers import lcd
    cur_dir = Path(__file__).parent
    with lcd(str(cur_dir)):
        api.local('yarn')
        api.local('yarn run build')
    with (cur_dir / "bci_demo_ssvep/gui/bci_demo_ssvep_resources.qrc").open('w') as resource_file:
        resource_file.write('<RCC><qresource prefix="/">')
        for f in os.listdir(str(cur_dir / 'bci_demo_ssvep/gui/dist')):
            resource_file.write('<file alias="ssvep/%s">dist/%s</file>' % (f, f))
        resource_file.write('</qresource></RCC>')


def add_pack_bci_demo(command):
    class PackBciDemo(command):
        def run(self):
            package_bci_demo()
            super().run()

    return PackBciDemo


cmd_classes['build_ui'] = add_pack_bci_demo(cmd_classes.get('build_ui', Command))
cmd_classes['sdist'] = add_pack_bci_demo(cmd_classes.get('sdist', sdist))

if __name__ == '__main__':
    setup(
        name='bci-demo-ssvep',
        cmdclass=cmd_classes,
        zip_safe=False,
        author='BrainTech',
        author_email='admin@braintech.pl',
        description='OBCI BCI SSVEP Demo',
        version=versioneer.get_version(),
        packages=find_packages(),
        include_package_data=True,
        install_requires=INSTALL_REQUIREMENTS['install'],
        setup_requires=INSTALL_REQUIREMENTS['setup'],
        entry_points={
            'console_scripts': [
                'bci_demo_ssvep = bci_demo_ssvep.entrypoint:run',
                'install_bci_demo_ssvep = bci_demo_ssvep.install:run',
            ],
        },
    )
