# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import threading
import time

from mock import MagicMock

from bci_demo_ssvep import config_generation
from bci_demo_ssvep.config_generation import SSVEPRandomAmplifierPeer  # noqa
from bci_demo_ssvep.demo_ssvep_app import SSVEPDemoExperiment
from bci_demo_ssvep.gui.ssvep_frontend import BlinkerController
from bci_demo_ssvep.peers.logic import SSVEPLogic # noqa
from bci_demo_ssvep.simple_ssvep.frontend import Frontend
from bci_ssvep.logic.ssvep_calibrator import SSVEPCalibrator
from bci_ssvep.logic.ssvep_metacalibrator import SSVEP_Metacalibrator

MONTAGE_CHANNELS = ["Random1", "Random2"]
ANALYSIS_CHANNELS = ["Random3", "Random4", "Random5"]


class _TestFrontend(Frontend):

    def __init__(self, loop=None):
        super().__init__(loop)
        self.decision_event = threading.Event()
        self.calibrated_event = threading.Event()

    async def display_prediction(self, field, feedback_duration, is_correct=True):
        await super().display_prediction(field, feedback_duration, is_correct)
        self._last_decision = field
        self.decision_event.set()

    async def calibration_finished(self):
        await super().calibration_finished()
        self.calibrated_event.set()

    def clear(self):
        pass


def test_ssvep_demo_experiment(mocker, fast_analysis):
    config = config_generation.generate(
        amplifier_id='RandomAmplifierWithSine',
        analysis_channels=ANALYSIS_CHANNELS,
        montage_channels=MONTAGE_CHANNELS,
        exclude_from_montage=ANALYSIS_CHANNELS[-1:],
        sampling_rate=500,
        filter_frequency=50,
        field_freqs_min=42,
        field_freqs_max=45,
    )
    config['amplifier']['samples_per_packet'] = 100
    FatBlinker = mocker.patch('braintech.drivers.blinker.fat_blinker.FatBlinker')
    blinker = FatBlinker.return_value
    bci_demo = MagicMock()
    bci_frontend = _TestFrontend()
    SSVEP_Metacalibrator.MINIMUM_EXAMPLE_COUNT = 1
    SSVEPCalibrator.TRIAL_TIME = 2
    SSVEPCalibrator.PREPARE_TIME = 0.2
    blinker_controler = BlinkerController(2)
    experiment = SSVEPDemoExperiment(config, bci_frontend, blinker_controler)
    experiment._as_subprocess = False
    experiment.start(config)
    amplifier = experiment._amplifier  # type: SSVEPRandomAmplifierPeer
    amplifier.ssvep_autocalibration_check_handle(1)
    logic = experiment._logic  # type: SSVEPLogic

    # CALIBRATION
    calib_finished = bci_frontend.calibrated_event
    blinking_started = threading.Event()
    blinker.start.side_effect = blinking_started.set
    experiment.start_calibration()
    time.sleep(0.5)
    bci_frontend.handle_key(bci_frontend.SPACE)
    assert blinking_started.wait()
    assert calib_finished.wait()
    amplifier.ssvep_autocalibration_check_handle(0)
    amplifier.amplifier_frequency_select_handle(0)
    time.sleep(0.2)
    experiment.finish()
    # BCI
    best_freqs = logic._calibrator.field_freqs
    prediction_displayed = bci_frontend.decision_event
    experiment.start_bci()
    amplifier.amplifier_frequency_select_handle(best_freqs[1])
    assert prediction_displayed.wait()
    assert bci_frontend._last_decision == 1
    prediction_displayed.clear()
    amplifier.amplifier_frequency_select_handle(best_freqs[0])
    assert prediction_displayed.wait()
    assert bci_frontend._last_decision == 0
    experiment.stop()
