import svg from './impedance.svg'
import global_settings from './settings'
import {on_gridstack_item_resize} from './charts'

settings =
  good_weak_border: 50
  weak_bad_border: 100
  good_color: 'hsla(120, 67%, 49%, 0.7)'
  weak_color: 'hsla(60, 67%, 49%, 0.7)'
  bad_color: 'hsla(0, 67%, 49%, 0.7)'
  electrodes:
    supported: ['Cz', 'C3', 'C4', 'P3', 'P4', 'Pz', 'O1', 'O2', 'M1', 'M2']
    circle_color: global_settings.electrodes.colors


initializeImpedance = (socket) ->
  $('impedance').each ->
    impedance = $(this)
    impedance.html(svg)
    bind_to_socket(socket, impedance)
    _bind_resize_event(impedance)

bind_to_socket = (socket, impedance)->
  socket.on 'impedances', (data) ->
    multiplier = 1
    for name in settings.electrodes.supported
      if name of data
        value = data[name]
        value = Math.round(value * multiplier)
        if value > 999
          text = '∞'
        else
          text = value
        fill = _count_color(value)
        stroke = settings.electrodes.circle_color[name]
      else
        text = ''
        fill = 'transparent'
        stroke = 'transparent'
      impedance.find("svg text.#{name}").text(text)
      impedance.find("svg circle.#{name}")
        .attr('fill', fill)
        .attr('stroke', stroke)


_count_color = (value) ->
  if value < settings.good_weak_border
    return settings.good_color
  else if value < settings.weak_bad_border
    return settings.weak_color
  else
    return settings.bad_color

_bind_resize_event = (impedance)->
  resize = ->
    impedance.find('svg').attr(height:0,width:0).attr(height:impedance.height(),width:"100%")
  on_gridstack_item_resize(impedance,resize)
  resize()


export default initializeImpedance
