const path = require('path');
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
const devMode = process.env.NODE_ENV !== 'production'
const CopyWebpackPlugin = require('copy-webpack-plugin')
const output = path.resolve(__dirname, 'bci_panel/dist')
module.exports = {
  context: path.resolve(__dirname, 'static/src'),
  entry: {
    app: './app.coffee',
  },
  output: {
    path: output,
    filename: '[name].bundle.js',
  },
  resolve: {
    modules: [path.resolve(__dirname, 'static/src'), path.resolve(__dirname, '../node_modules')],
    alias: {
      style: path.resolve(__dirname, 'static/style'),
      "gridstack.scss": "gridstack/src/gridstack.scss",
      'gridstack.jqueryUI': "gridstack/dist/gridstack.jQueryUI.js",
      'jquery-ui': 'jquery-ui/ui',
      'c3.css': "c3/c3.css",

    },
    extensions: ['.coffee', '.js']
  },
  devtool: 'cheap-module-eval-source-map',
  module: {
    rules: [
      {
        test: /\.coffee$/i,
        use: [{
          loader: 'coffee-loader',
          options: {
            transpile: {
              presets: ['env']
            }
          }
        }]
      },
      {
        test: /\.(sass|scss)$/i,
        use: [
          devMode ? 'style-loader' : MiniCssExtractPlugin.loader,
          'css-loader',
          {
            loader: 'postcss-loader', // Run post css actions
            options: {
              plugins: function () { // post css plugins, can be exported to postcss.config.js
                return [
                  require('autoprefixer')
                ];
              }
            },
          },
          'sass-loader',
        ]
      },
      {
        test: /\.css$/i,
        use: [
          devMode ? 'style-loader' : MiniCssExtractPlugin.loader,
          'css-loader',
          {
            loader: 'postcss-loader', // Run post css actions
            options: {
              plugins: function () { // post css plugins, can be exported to postcss.config.js
                return [
                  require('autoprefixer')
                ];
              }
            },
          }
        ]
      },
      {
        test: /test\.js$/,
        use: 'mocha-loader',
        exclude: /node_modules/,
      }, {
        test: /\.svg$/,
        loader: 'svg-inline-loader'
      },
      {
        test: /\.(jpg|png)$/,
        use: {
          loader: "url-loader",
          options: {
            limit: 25000,
          },
        },
      },
    ]
  },
  devServer: {
    contentBase: './static',
    // publicPath: '/bci_panel',
    // hot: true
  },
  plugins: [
    new CleanWebpackPlugin([output]),
    new HtmlWebpackPlugin({
      title: 'BCI Control Panel',
      template: path.resolve(__dirname, 'static/index.html'),
    }),
    new MiniCssExtractPlugin({
      // Options similar to the same options in webpackOptions.output
      // both options are optional
      filename: devMode ? '[name].css' : '[name].[hash].css',
      chunkFilename: devMode ? '[id].css' : '[id].[hash].css',
    }),
    new webpack.NamedModulesPlugin(),
    new webpack.ProvidePlugin({
      $: "jquery",
      jQuery: "jquery"
    }),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
    new CopyWebpackPlugin([{from: '../img', to: 'img'}], {}),
    // new BundleAnalyzerPlugin(),

  ],
  optimization: {
    splitChunks: {
      cacheGroups: {
        commons: {
          test: /[\\/]node_modules[\\/]/,
          name: 'vendor',
          chunks: 'all'
        }
      }
    }
  }
};
