# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

from obci_ugm import utils


class FrontendParameters:
    TOP_FIELD_ID = '-1'

    def __init__(self, status_text: str, field_statuses: [str], field_freqs: [int],
                 pause_text: str, trials_count: int, questions: [str],
                 questions_correct_answers_active_field_ids: [str],
                 instruction_type_audio: bool, question_type_audio: bool,
                 feedback_type_audio: bool, feedback_texts: [str], bye_text: str,
                 first_feedback_field_id: int,
                 thinking_pause_after_question_length: float):
        self.status_text = status_text
        self.field_statuses = field_statuses
        self.field_freqs = field_freqs
        self.pause_text = pause_text
        self.trials_count = trials_count
        self.questions = questions
        self.questions_correct_answers_active_field_ids = (
            questions_correct_answers_active_field_ids
        )
        self.instruction_type_audio = instruction_type_audio
        self.question_type_audio = question_type_audio
        self.feedback_type_audio = feedback_type_audio
        self.feedback_texts = feedback_texts
        self.bye_text = bye_text
        self.first_feedback_field_id = first_feedback_field_id
        self.thinking_pause_after_question_length = thinking_pause_after_question_length

    @property
    def statuses(self):
        statuses = {str(nr): text for nr, text in enumerate(self.field_statuses)}
        statuses[self.TOP_FIELD_ID] = self.status_text
        return statuses

    @property
    def frequency_to_field_map(self):
        return {v: k for k, v in self.field_to_frequency_map.items()}

    @property
    def field_to_frequency_map(self):
        map = {str(nr): self.field_freqs[nr]
               for nr, text in enumerate(self.field_statuses)}
        map[self.TOP_FIELD_ID] = 0
        return map

    @property
    def field_to_feedback_text(self) -> {str: str}:
        assert len(self.feedback_texts) == len(self.field_statuses) + 2
        per_normal_field = {str(i): self.feedback_texts[i]
                            for i in range(len(self.field_statuses))}
        special = {self.TOP_FIELD_ID: self.feedback_texts[-2],
                   None: self.feedback_texts[-1]}
        return {**per_normal_field, **special}

    @property
    def question_to_answer_field_id(self) -> {str: str}:
        return {self.questions[i]: self.questions_correct_answers_active_field_ids[i]
                for i in range(len(self.questions))}

    @utils.listify
    def get_speech_texts(self) -> [str]:
        if self.instruction_type_audio:
            yield self.status_text
            yield from self.field_statuses
        if self.question_type_audio:
            yield from self.questions
        if self.feedback_type_audio:
            yield from self.feedback_texts

    def copy(self):
        return self.__class__(
            status_text=self.status_text,
            field_statuses=self.field_statuses,
            field_freqs=self.field_freqs,
            pause_text=self.pause_text,
            trials_count=self.trials_count,
            questions=self.questions,
            questions_correct_answers_active_field_ids=self.questions_correct_answers_active_field_ids,
            instruction_type_audio=self.instruction_type_audio,
            question_type_audio=self.question_type_audio,
            feedback_type_audio=self.feedback_type_audio,
            feedback_texts=self.feedback_texts,
            bye_text=self.bye_text,
            first_feedback_field_id=self.first_feedback_field_id,
            thinking_pause_after_question_length=self.thinking_pause_after_question_length,
        )
