# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import time


class SSVEPCalibrationResult:
    def __init__(self, calibration_buffer, ssvep_decider):
        self.creation_time = time.time()
        self.ssvep_decider = ssvep_decider

        self.roc_auc = 0
        n = 0
        for i in self.ssvep_decider.thresholds.values():
            try:
                self.roc_auc += i.abs_auroc
                n += 1
            except TypeError:
                pass
            try:
                self.roc_auc += i.diff_auroc  # might be none
                n += 1
            except TypeError:
                pass
        try:
            self.roc_auc /= n
        except ZeroDivisionError:
            self.roc_auc = 0

        self.calibration_buffer = calibration_buffer

    def __repr__(self):
        return "<{} ROC: {}, N: {}>".format(self.__class__.__name__, self.roc_auc, self.get_minimum_example_count())

    def get_available_freqs(self):
        return self.ssvep_decider.stimulation_freqs

    def get_minimum_roc(self):
        rocs = []
        for f in self.get_available_freqs():
            freq_thr = self.ssvep_decider.thresholds[f]
            rocs.append(freq_thr.diff_auroc)
            rocs.append(freq_thr.abs_auroc)
        return min(rocs)

    def roc_for_freq(self, f):
        freq_thr = self.ssvep_decider.thresholds[f]
        return (freq_thr.abs_auroc + freq_thr.diff_auroc) / 2

    def rocs_for_freqs(self):
        return [self.roc_for_freq(f) for f in self.get_available_freqs()]

    def get_minimum_example_count(self):
        lens = []
        for value in self.calibration_buffer.values():
            lens.append(len(value))
        return min(lens)

    def get_mean_example_count_per_frequency(self):
        lens = []
        for value in self.calibration_buffer.values():
            lens.append(len(value))
        return sum(lens) / len(lens)

    def is_valid(self):
        for i in self.ssvep_decider.thresholds.values():
            for measure in i.__dict__.values():
                if measure is None:
                    return False
        return True
