# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import logging
from functools import wraps

import PySide2
from PySide2 import QtWebEngineWidgets, QtCore
from PySide2.QtCore import QJsonValue
from PySide2.QtGui import QDesktopServices
from PySide2.QtWebEngineWidgets import QWebEngineSettings

logger = logging.getLogger(__name__)


class WebEnginePage(QtWebEngineWidgets.QWebEnginePage):
    def javaScriptConsoleMessage(self, log_level, msg, line, file):
        if log_level == self.JavaScriptConsoleMessageLevel.InfoMessageLevel:
            level = logging.INFO
        elif log_level == self.JavaScriptConsoleMessageLevel.WarningMessageLevel:
            level = logging.WARNING
        elif log_level == self.JavaScriptConsoleMessageLevel.ErrorMessageLevel:
            level = logging.ERROR
        else:
            level = logging.DEBUG
        logger.log(level, msg + " %s:%d", file, line)

    def acceptNavigationRequest(self, url: PySide2.QtCore.QUrl,
                                type: PySide2.QtWebEngineWidgets.QWebEnginePage.NavigationType, isMainFrame: bool):
        if type == QtWebEngineWidgets.QWebEnginePage.NavigationTypeLinkClicked:
            if url.host() != self.url().host() or url.port() != self.url().port():
                QDesktopServices.openUrl(url)
                return False
        return True


class WebView(QtWebEngineWidgets.QWebEngineView):
    window_event = QtCore.Signal(str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setPage(WebEnginePage(self))
        self.page().settings().setAttribute(QWebEngineSettings.LocalContentCanAccessRemoteUrls, True)
        self.page().settings().setAttribute(QWebEngineSettings.LocalContentCanAccessFileUrls, True)

    def changeEvent(self, event: QtCore.QEvent):
        if event.type() == QtCore.QEvent.WindowStateChange:
            if self.isMaximized():
                self.window_event.emit('maximized')
            elif self.isMinimized():
                self.window_event.emit('minimized')
            else:
                self.window_event.emit('normalized')
        super(WebView, self).changeEvent(event)

    def closeEvent(self, event):
        self.window_event.emit('closed')


def unwrap_json(func):
    @wraps(func)
    def wrapper(*args):
        unwrapped_args = []
        for a in args:
            if isinstance(a, QJsonValue):
                unwrapped_args.append(a.toVariant())
            else:
                unwrapped_args.append(a)
        return func(*unwrapped_args)

    return wrapper
