import cmd
import logging
from contextlib import contextmanager

from obci_demo.obci_demo_experiment import OBCIDemoExperiment, InvalidState

logger = logging.getLogger(__name__)


def run(config):
    Shell(config=config).cmdloop()


@contextmanager
def catch_invalid():
    try:
        yield
    except InvalidState as ex:
        logger.warning(str(ex))


class Shell(cmd.Cmd):
    intro = '\nWelcome! Type "help" for help.'
    prompt = '(obci_demo)$ '

    def __init__(self, config, *args, **kwargs):
        self._experiment = OBCIDemoExperiment(config)
        self._experiment.start(config)
        super().__init__(*args, **kwargs)

    def do_calibration(self, _):
        """
        Start P300 calibration job.

        You can end it by issuing the "stop" command.
        """
        with catch_invalid():
            logger.info('Starting calibration...')
            self._experiment.start_calibration()
            logger.info('Okay, calibration is running.')

    def do_communication(self, _):
        """
        Start P300 communication job. Requires prior calibration.

        You can end it by issuing the "stop" command.
        """
        with catch_invalid():
            logger.info('Starting communication...')
            self._experiment.start_bci()
            logger.info('Okay, communication is running.')

    def do_stop(self, _):
        """
        Stop the background job. Waits for it to finish.
        """
        with catch_invalid():
            self._experiment.finish()

    def do_EOF(self, _):
        """
        Exit (by clicking Ctrl+D).
        """
        return self.do_exit(_)

    def do_exit(self, _):
        """
        Exit.
        """
        with catch_invalid():
            self._experiment.shutdown()
        return True

    def emptyline(self):
        pass
