import os
from pathlib import Path

from setuptools import find_packages, setup, Command
from setuptools.command.sdist import sdist

import versioneer

INSTALL_REQUIREMENTS = {
    'install': [
        'coloredlogs==10.0',
        'PySide2',
        'obci_ugm',
        'obci-p300'
    ],
    'test': [
        'socketIO-client'
    ]
}
try:
    from devops.setup import get_cmd_class

    cmd_classes = get_cmd_class()
except ImportError:
    print("devops not installed yet, can't run build_ui.")
    cmd_classes = versioneer.get_cmdclass()


def package_bci_demo():
    from fabric import api
    from fabric.context_managers import lcd
    cur_dir = Path(__file__).parent
    with lcd(str(cur_dir)):
        api.local('yarn')
        api.local('yarn run build')
    with (cur_dir / "obci_demo/gui/bci_demo_resources.qrc").open('w') as resource_file:
        resource_file.write('<RCC><qresource prefix="/">')
        for f in os.listdir(str(cur_dir / 'obci_demo/gui/dist')):
            resource_file.write('<file alias="demo/%s">dist/%s</file>' % (f, f))
        resource_file.write('</qresource></RCC>')


def add_pack_bci_demo(command):
    class PackBciDemo(command):
        def run(self):
            package_bci_demo()
            super().run()

    return PackBciDemo


cmd_classes['sdist'] = add_pack_bci_demo(cmd_classes.get('sdist', sdist))
cmd_classes['build_ui'] = add_pack_bci_demo(cmd_classes.get('build_ui', Command))
if __name__ == '__main__':
    setup(
        name='obci-demo',
        version=versioneer.get_version(),
        zip_safe=False,
        cmdclass=cmd_classes,
        author='BrainTech',
        author_email='admin@braintech.pl',
        description='BCI P300 Demo',
        packages=find_packages(),
        include_package_data=True,
        install_requires=INSTALL_REQUIREMENTS['install'],
        entry_points={
            'console_scripts': [
                'bci_demo_p300 = obci_demo.entrypoint:run',
                'install_bci_demo = obci_demo.install:run',
            ],
        },
    )
