import 'bootstrap-select'
import style from 'bootstrap-select/sass/bootstrap-select.scss'
$.fn.selectpicker.Constructor.BootstrapVersion = '4'

class BCIConfig
  constructor: (form_element, options, start_callback, stop_callback) ->
    @_options = options
    @_modal = form_element.parents('.modal')
    @_form = form_element
    @_start_callback = start_callback
    @_stop_callback = stop_callback
    @_amplifier = @_form.find('#amplifier')
    @_amplifier.empty()
    for amp_id,amp of @_options
      @_amplifier.append("<option value='#{amp_id}'>#{amp.name}</option>")
    @_amplifier.selectpicker('refresh')

    @_channels = @_form.find("#channels")
    @_montage_channels = @_form.find("#montage_channels")
    @_excluded_channels = @_form.find("#exclude_from_montage")
    @_set_amplifier(_.keys(options)[0])
    @_start = @_form.find("#start_amplifier")
    @_stop = @_form.find("#stop_amplifier")
    @_form.on 'submit', @_on_submit
    @_stop.on 'click', @_on_stop
    @_amplifier.on 'change', (e)=> @_set_amplifier($(e.target).val())
    @_channels.on 'change', @_channels_changed
    @_form.find('input,select').on 'change', =>
      @_form.removeClass('was-validated')

  _on_submit: (event) =>
    event.preventDefault()
    event.stopPropagation()
    @_form.addClass('was-validated')
    if @_form[0].checkValidity() is false
      return
    config = @_get_config()
    console.log config
    @_form.find('input,select,button').prop('disabled', true)
    start = @_start_callback(config)
    start.done =>
      @_stop.prop('disabled', false)
      @_modal.modal('hide')
    start.fail @_enable_start

  _get_config: ->
    return config =
      analysis_channels: @_get_selected(@_channels)
      exclude_from_montage: @_get_selected(@_excluded_channels)
      montage_channels: @_get_selected(@_montage_channels)
      amplifier_id: @_amplifier.val()
      sampling_rate: parseFloat(@_form.find("#sampling_rate").val())
      filter_frequency: parseFloat(@_form.find("[name=filtering]:checked").val())


  _enable_start: =>
    @_form.find('input,select,button').prop('disabled', false)
    @_stop.prop('disabled', true)

  _on_stop: =>
    @_stop.prop('disabled', true)
    stop = @_stop_callback()
    stop.done @_enable_start
    stop.fail =>
      @_stop.prop('disabled', false)

  _get_selected: (multiselect) ->
    result = []
    multiselect.children(":selected").each ->
      result.push($(this).val())
    return result

  _set_amplifier: (amplifier_id) =>
    amp_config = @_options[amplifier_id]
    sampling_rates = @_form.find('#sampling_rate')
    sampling_rates.empty()

    for s_r in amp_config.sampling_rates
      sampling_rates.append "<option value='#{s_r}'>#{s_r} Hz</option>"
    sampling_rates.selectpicker('refresh')
    channels = @_channels
    montage_channels = @_form.find('#montage_channels')
    montage_channels.empty()
    channels.empty()
    for ch in amp_config.channels
      channels.append("<option value='#{ch}'>#{ch}</option>")
      montage_channels.append("<option value='#{ch}'>#{ch}</option>")
    montage_channels.selectpicker('refresh')
    channels.selectpicker('refresh')
    @_form.find("#exclude_from_montage").empty()
    @_amp_config = amp_config

  _channels_changed: =>
    excluded = @_get_selected(@_excluded_channels)
    montage = @_get_selected(@_montage_channels)
    selected = @_get_selected(@_channels)
    @_excluded_channels.empty()
    @_montage_channels.empty()
    for s in selected
      @_excluded_channels.append("<option value='#{s}' #{if s in excluded then 'selected' else ''}>#{s}</option>")
    for s in @_amp_config.channels
      if s not in selected
        @_montage_channels.append("<option value='#{s}' #{if s in montage then 'selected' else ''}>#{s}</option>")
    @_excluded_channels.selectpicker('refresh')
    @_montage_channels.selectpicker('refresh')

  show: =>
    @_enable_start()
    @_form.removeClass('was-validated')
    @_modal.modal(keyboard:false, backdrop:'static')

export default BCIConfig
