# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import abc
import copy


class AbstractClassifier(metaclass=abc.ABCMeta):
    """Base class for all classifiers."""

    def __init__(self):
        pass

    @abc.abstractmethod
    def classify(self, chunk):
        """
        Compute set of classification probabilities for a given chunk of data.

        :param: chunk: numpy 2D data array (channels × samples)

        :return:dict. dictionary of {target: probability},
                or None if classification could not be performed
        """
        pass

    def clone(self):
        """
        Return an independent copy of this classifier.

        Can be overriden to implement more efficient cloning,

        :return: AbstractClassifier. copy of this classifier
        """
        return copy.copy(self)

    @abc.abstractmethod
    def learn(self, chunk, target):
        """Learn that given chunk of data represents given target.

        Args:
            chunk: numpy 2D data array (channels × samples)
            target: name of the target
        """
        pass
