# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import pytest

from braintech.obci.core import utils
from braintech.obci.core.broker.broker import Broker
from braintech.obci.experiment.peers.control.config_server import ConfigServer
from braintech.obci.experiment.peers.drivers.amplifiers.random_amplifier_peer import RandomAmplifierPeer
from obci_p300 import p300_peer
from braintech.obci.core.utils import yield_then_shutdown, get_peer_config_file_path


@pytest.fixture(scope='module')
def broker():
    yield from yield_then_shutdown(Broker())


@pytest.fixture(scope='module')
def config_server(broker):
    yield from yield_then_shutdown(
        ConfigServer(
            broker.broker_ip,
            peer_id='config_server',
            base_config_file=get_peer_config_file_path(ConfigServer),
        )
    )


@pytest.fixture(scope='module')
def amplifier(broker, config_server):
    config = {
        'local_params': {
            'autostart': '1',
            'autoshutdown': '0',
            'active_channels': '1;2;3;4',
            'channel_names': 'O1;O2;Pz;Cz',
        },
    }
    peer = create_peer(RandomAmplifierPeer, broker, name='amplifier',
                       dependencies=[config_server], config=config)
    yield from utils.yield_then_shutdown(peer)


@pytest.fixture
def p300_analysis_peer(broker, config_server, amplifier):
    calibration_local_params = {
        'channels_for_classification': 'O1;O2;Pz;Cz',
        'channels_for_montage': '',
        'offline_learning': '0',
        'downsample_to': '24',
        'calibration_field_index': '0',
    }
    config = {
        'local_params': {
            'autostart': '1',
            'autoshutdown': '0',
            **calibration_local_params,
        },
        'launch_dependencies': {
            'signal_source': amplifier.id,
        },
    }
    peer = create_peer(p300_peer.P300Peer, broker, name='p300_analysis',
                       dependencies=[config_server, amplifier], config=config)
    yield from utils.yield_then_shutdown(peer)


def create_peer(cls, broker, name, dependencies, config=None):
    if config is None:
        config = {}
    for dependency in dependencies:
        utils.wait_until_peers_ready([broker, dependency], timeout=3)
    config_path = utils.get_peer_config_file_path(cls)
    return cls(
        urls=broker.broker_ip,
        peer_id='{}_id'.format(name),
        peer_name=name,
        base_config_file=config_path,
        **config,
    )
