# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import time

from braintech.obci.core.utils.message_helpers import send_tag


class TagsSender:
    def __init__(self, communication_peer):
        self._communication_peer = communication_peer

    async def send_blink(self, start, duration, index, current_target):
        end = start + duration
        description = {'index': index, 'target': current_target}
        await send_tag(self._communication_peer, start, end, 'blink', description)

    async def send_text_instruction_start(self):
        now = time.time()
        description = {}
        await send_tag(self._communication_peer, now, now,
                       'text_instruction_start', description)

    async def send_audio_instruction_start(self, duration):
        now = time.time()
        description = {'duration_in_seconds': duration}
        await send_tag(self._communication_peer, now, now,
                       'audio_instruction_start', description)


class ExperimentTagsSender(TagsSender):
    async def send_decision(self, decision, answer=None):
        if answer is not None:
            decision['p_tag_desc']['decision_correct'] = answer
        await send_tag(self._communication_peer, **decision)

    async def send_text_question_start(self):
        now = time.time()
        description = {}
        await send_tag(self._communication_peer, now, now,
                       'text_question_start', description)

    async def send_audio_question_start(self, duration):
        now = time.time()
        description = {'duration_in_seconds': duration}
        await send_tag(self._communication_peer, now, now,
                       'audio_question_start', description)

    async def send_visual_feedback_start(self):
        now = time.time()
        description = {}
        await send_tag(self._communication_peer, now, now,
                       'visual_feedback_start', description)

    async def send_audio_feedback_start(self, duration):
        now = time.time()
        description = {'duration_in_seconds': duration}
        await send_tag(self._communication_peer, now, now,
                       'audio_feedback_start', description)
