# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
import numpy as np
import pylab as pb
import pytest

from obci_p300.p300_classm import P300EasyClassifier


VISUAL_DEBUG = False


def _targets_nontargets_generator(size=20, snr=0.42):
    target = True
    p300_waveform = np.kaiser(size, beta=20) * snr
    while True:
        sample = np.random.normal(size=size)
        if target:
            sample += p300_waveform
        target = not target
        yield (sample, 'target' if target else 'nontarget')


@pytest.mark.timeout(3600)
def test_cv_and_classification():
    clf = P300EasyClassifier(learn_every=5, targetFs=24, use_linear_proba=True, clf=None)

    # without debug 10 - so the test wouldn't be flaky
    gen = _targets_nontargets_generator(snr=0.42 if VISUAL_DEBUG else 10)

    for i in range(200 if VISUAL_DEBUG else 70):
        chunk, target = next(gen)
        clf.learn(chunk, target)

    if VISUAL_DEBUG:
        pb.figure()
        for i in range(len(clf.n_averaged_rocs)):
            pb.plot(clf.rocs[i][1], clf.rocs[i][0], label='N={}'.format(clf.n_averaged_rocs[i]))
            pb.xlim([-0.01, 1.0])
            pb.ylim([0, 1.01])
        pb.legend()
        pb.figure()
        pb.plot(clf.n_averaged_rocs, clf.roc_aucs, '-o')
        pb.title('N average vs AUC')
        pb.ylim([0, 1.01])

        pb.figure()
        pb.title("AUC (no averaging) history")
        pb.plot(clf.history_n, clf.AUC_history, '-o')
        pb.ylim([0, 1.01])
        pb.show()

    for i in clf.roc_aucs:
        assert i > 0.9

    for i in range(100):
        chunk, target = next(gen)
        if target == 'target':
            assert clf.classify(chunk) >= 0.5
        else:
            assert clf.classify(chunk) < 0.5
