## BCI framework

### Status of development build

[![build status](https://git.braintech.pl/brain/openbci/badges/development/build.svg)](https://git.braintech.pl/brain/openbci/commits/development) [![coverage report](https://git.braintech.pl/brain/openbci/badges/development/coverage.svg)](https://git.braintech.pl/brain/openbci/commits/development)

### Description

BCI framework is a platform for Brain Computer Interfaces. This software contains tools which allow you to build a complete Brain Computer Interface based on EEG, and perform some experiments collecting EEG and other biomedical data signals.

These tools are:

 * tools for communication with some EEG hardware (TMSi, Braintronics, and openEEG)
 * tools for displaying and storing the EEG (and other biomedical time series) signal
 * tools for creating "bindings" or "use scenarios" for some 3rd party software for performing psychological experiments (e-prime, psychopy, visionEGG). This means, that if you prepare an experiment in some of mentioned software, you will be able to perform this experiment, and store the EEG data, with necessary tags using BCI-Framework.

### Documentation

[Nightly](http://deb-nightly.braintech.pl/docs/), [Testing](http://deb-testing.braintech.pl/docs/), [Stable](http://deb-stable.braintech.pl/docs/)

Login credentials are available [here](https://redmine.titanis.pl/projects/brain4/wiki/Has%C5%82a_do_repozytori%C3%B3w).

Adding IP cameras to server: [Camera manual](https://redmine.titanis.pl/projects/brain4/wiki/Instrukcja_pod%C5%82%C4%85czenia_kamery_IP).

### Installation

BCI framework can be installed from DEB packages. To add DEB packages repository
execute following command replacing placeholders with suitable values:

* `{UBUNTU_RELEASE}` - currently only `xenial` (Ubuntu 16.04) is supported
* `{HTTP_USER}` - user name for selected release channel (get from [Titanis Redmine Wiki](https://redmine.titanis.pl/projects/brain4/wiki/Has%C5%82a_do_repozytori%C3%B3w))
* `{HTTP_PASSWD}` - password for selected release channel (get from [Titanis Redmine Wiki](https://redmine.titanis.pl/projects/brain4/wiki/Has%C5%82a_do_repozytori%C3%B3w))

```
sudo sh -c "echo deb http://{HTTP_USER}:{HTTP_PASSWD}@deb-stable.braintech.pl/{UBUNTU_RELEASE} / > /etc/apt/sources.list.d/braintech-stable-{UBUNTU_RELEASE}.list"
```

If you want to install testing or nightly builds you need to *additionally*
add following repository:

* `{RELEASE_CHANNEL}` - release channel: `nightly` or `testing`

```
sudo sh -c "echo deb http://{HTTP_USER}:{HTTP_PASSWD}@deb-{RELEASE_CHANNEL}.braintech.pl/{UBUNTU_RELEASE} / > /etc/apt/sources.list.d/braintech-{RELEASE_CHANNEL}-{UBUNTU_RELEASE}.list"
```

Or click [here](https://redmine.titanis.pl/projects/brain4/wiki/Has%C5%82a_do_repozytori%C3%B3w) for a list of predefined commands.

Add Braintech's package signing key

```
sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys 1452AFDF
```

Downloads the package lists from the repositories

```
sudo apt-get update
```

Install BCI framework

```
sudo apt-get install obci2
```

### Development/Running from source

#### Prerequisites

BCI framework requires Python 3.5.1+ and PyZmq 16.0+.

To develop BCI framework you will need at least `git`, `pip`, `pyqt5-dev-tools` (or `pyqt5` installed using pip), and `setuptools`
installed. Using `pyvenv` is strongly recommended for developers.

VideoEEG requires liveMedia libraries, headers and a C compliler, and without them setup.py will fail, additionally mkvmergeis required.

P300 calibration uses pyaudio (portaudio.h), so portaudio19-dev is required.

We recommend using [PyCharm](https://www.jetbrains.com/pycharm/) on Linux and Visual Studio 2015 Community or higher with [PTVS](https://microsoft.github.io/PTVS/) on Windows.

We assume pip for Python 3 will be named `pip3` (as on Ubuntu). On Windows and inside virtual environments pip is usually usually `pip`. `pip3` will be told to install locally, with `--user` flag.

If you use Ubuntu (or you are a tester):

* You can run `sudo apt install git python3-pip python3-setuptools python3-venv pyqt5-dev-tools liblivemedia50 liblivemedia-dev mkvtoolnix ffmpeg libbluetooth-dev build-essential portaudio19-dev ` to install mentioned prerequisites on Linux
* `pip3` with `--user` flag installs binaries in `~/.local/bin` directory. By default this directory is not included in `$PATH`.
  Add following lines to your `.profile` or `.bashrc` to make pip-installed binaries runnable from `$PATH`
  ```
  if [ -d "$HOME/.local/bin" ] ; then PATH="$HOME/.local/bin:$PATH" ; fi
  ```

HINT: You can run `flake8 --install-hook=git` inside your repository to enable flake8 checking before each commit. We recommend you to add `export FLAKE8_STRICT=true` line to your default profile to abort commit if errors are found.

#### Downloading source code

Using HTTP authentication:

```
git clone https://gitlab.com/braintech/openbci.git
```

Or if you had configured SSH access to GitLab:

```
git clone git@gitlab.com:braintech/openbci.git
```


#### Installing dependencies

```
sudo pip3 install fabric3
```

#### Installing local, editable copy

Basic command for installing all obci packages and python dependencies is:

```
fab setup_dev
```

Get authorization variables for the script below from: https://redmine.titanis.pl/projects/brain4/wiki/Hasła_do_repozytoriów

```
export HTTP_USER=
export HTTP_PASSWD=
```

Go to the repository root directory.

For running tests locally run:

```
sudo fab --fabfile=devops/ci/install.py setup_obci_tests_stage
```

For building Sphinx-based documentation locally run:

```
sudo fab --fabfile=devops/ci/install.py setup_obci_gen_docs
```

Other possible dependency sets:

* setup_obci_builddebobci
* setup_obci_codestyle_stage
* setup_obci_deploy_stage
* setup_obci_gen_docs
* setup_obci_sign_stage

Please refer to devops/ci/install.py for details.


#### Uninstalling

BCI framework can be uninstalled using

```
pip3 uninstall obci
```


### Running tests

Install dependencies using:
```
pip3 install --user -e .[test]
```

Run tests:
```
pytest
```

Running tests create `.eggs` directory inside repository root. You can safely delete this directory after running tests.

### Building documentation

Install dependencies using:
```
pip3 install --user -e .[docs]
```

Build documentation:
```
make --directory=docs html
```

Documentation will be in `docs/_build/html` directory.

## Working with submodules

We use git submodules to insert another git project into a git project. In this case we have `devops` subproject.

Basic commands (ran from the main project):
* Initialization (once after cloning this repo): `git submodule init`
* Checking out: `git submodule update` - will checkout the proper commit of the submodule, possibly pulling from repo.
* Check which commit is current: `git submodule status`

### How it works

Git tracks 3 things for each submodule:
* local path - where it should be locally (`path` in `.gitmodules` file)
* remote repo location - where it should pull from (`url` in `.gitmodules` file)
* current commit hash - which commit should be checked out (stored in git database in `.git` folder - if changed, should be commited)

When you clone the main project you get an empty folder for submodules. For some fantastic reason you have to first `init` it, `update` alone won't do. Anyway `git submodule update` is convenient, because it pulls the subproject when in doubt.

### Changing used submodule version

In short:
1. `cd devops`
2. `git checkout ...` (you can `git pull` and any other normal command, it's a fully functioning repo as if it wasn't a submodule)
3. `cd ..`
4. `git add -p devops` - you'll see the diff and can confirm.
5. `git commit ...`

Description:

Current commit in a submodule is stored in git. It's commited. When you go to the submodule directory and check out some wondrous other commit, then go back to the main repo to marvel at what you've done, `git diff` may look like this:

```bash
staring@abyss:~/dev/null$ git diff
diff --git a/devops b/devops
index 99061c8..b2d84bb 160000
--- a/devops
+++ b/devops
@@ -1 +1 @@
-Subproject commit 99061c886ec078a43a0786836a7310dd9e6002af
+Subproject commit b2d84bb528bdb4869d7743294673b7641a3c7f2c
```

It compares the saved hash with what's really going on in the submodule directory.

Now you can `git add` it. Congratulations!

## Report logical errors
Logical errors - bugs that do not cause traceback and are not reported by Sentry.

To send error report run commend:
```
obci report_error
```

If you want configure Redmine install redmine (it's not required in deb package. Tester/Developer should install it manually.)
```
pip install python-redmine
```
Next add redmine's url and token into file ~/.obci/main_config/:
url=https://redmine.titanis.pl
token= You can find it in your redmine account: API access key (or in polish: Klucz dostępu do API)

## License

BCI framework is licensed under the terms of the GNU GPL version 3.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
